function net = experiment_learnTexture(config, net)

category_path = '../Image/texture/';
category_lists = dir([category_path, '*']);
valid_category = true(1, length(category_lists));
for i = 1:length(category_lists)
    if category_lists(i).name(1) == '.' || ~category_lists(i).isdir
       valid_category(i) = false; 
    end
end

category_lists = category_lists(valid_category);

category_lists = [];
category_lists(1).name = 'grass1';

for ii = 1:length(category_lists)
    close all;
    gpuDevice();
    
    fprintf('Learning category: %s\n', category_lists(ii).name);
    if nargin < 2
        [config, net] = frame_config(category_lists(ii).name, 'dense', 16, 'texture');
    end
    
    %% Step 1: compute training sample averages
    imgCell = read_images(config, net);
    img_mat = imgcell2mat(imgCell);
    
    numBatch = ceil(size(img_mat, 4) / config.BatchSize);
    rHat_batch = cell(numBatch, 1);
    for i = 1:numBatch
        start_idx = 1 + (i-1) * config.BatchSize;
        end_idx = min(config.BatchSize + start_idx - 1, size(img_mat, 4));
        rHat_batch{i} = filter_response(net, img_mat(:,:,:,start_idx:end_idx));
    end
    
    %% Step 2: optimize the exponential model
    syn_mat = zeros([config.sx, config.sy, 3, ...
        config.nTileRow * config.nTileCol], 'single');
    
    switch config.model_type
        case 'dense'
            net = train_texture_dense(config, net, syn_mat, rHat_batch);
        otherwise
            fprintf('Not supported');
    end
end