function [config, net] = frame_config(category, model_type, layer, exp_type)

% we only support gpu

if nargin < 2
    model_type = 'sparse';
end

if nargin < 1
    category = 'cat';
end

if nargin < 3
    layer = 11;
end

if nargin < 4
    exp_type = 'none';
end

learningRate = 0.1 * ones(1, 37);
learningRate(34) = 0.8;
learningRate(30) = 0.1;
learningRate(29) = 0.1;
learningRate(24:28) = 0.005;
learningRate(15:23) = 0.001;
learningRate(10:14) = 0.0006;
learningRate(9) = 0.0002;
learningRate(7) = 0.0003;
learningRate(4) = 0.0001;
learningRate(2) = 0.0001;

% category name
config.categoryName = category;

% model type: can either be dense model or spase model
config.model_type = model_type;

% experiment type: currently support 'texture', 'codebook'
%          
config.exp_type = exp_type;

% image path: where the dataset locates
switch lower(exp_type)
    case {'texture', 'codebook'}
        config.inPath = ['../Image/', exp_type, '/' config.categoryName '/'];
    otherwise
        config.inPath = ['../Image/object/' config.categoryName '/'];
end

% 3rd party path: where the matconvnn locates
config.matconvv_path = '../matconvnet-1.0-beta13/';

% model path: where the deep learning model locates
config.model_path = '../model/';

if strcmpi(config.exp_type, 'compile') == 1
    return;
end

% deep learning model name: default we use vgg-16
% for more model, please visit
% http://www.vlfeat.org/matconvnet/pretrained/ for more information
config.model_name = 'imagenet-vgg-verydeep-16.mat';

% parameter for synthesis
% nTileRow \times nTileCol defines the number of paralle chains
% right now, we currently support square chains, e.g. 2*2, 6*6, 10*10 ...
config.nTileRow = 2; 
config.nTileCol = 2;

% standard deviation for reference model q(I/sigma^2)
% no need to change. 30 - 100 recommended
config.refsig = 100;

% update rate for langevin dynamics
% no need to change.
config.Delta = 0.0112 * 10;

% result file: no need to change
config.working_folder = ['./working/', [config.categoryName, '_', config.model_type, '_layer_', num2str(layer)], '/'];
config.Synfolder = ['./synthesiedImage/', [config.categoryName, '_', config.model_type, '_layer_', num2str(layer)], '/'];

if strcmp(exp_type, 'none') ~= 1
    config.working_folder = [config.working_folder(1:end-1), '_',  exp_type, '/'];
    config.Synfolder = [config.Synfolder(1:end-1), '_',  exp_type, '/'];
end

% create directory
if ~exist('./synthesiedImage/', 'dir')
   mkdir('./synthesiedImage/') 
end

if ~exist(config.Synfolder, 'dir')
   mkdir(config.Synfolder);
end

if ~exist('./working/', 'dir')
    mkdir('./working/')
end

if ~exist(config.working_folder, 'dir')
    mkdir(config.working_folder);
end


% number of filters and training stage
config.interval = 20; % control how many iterations we need to wait to select next wavlet 
config.numWavelet = 300; % # of selected wavelets, default 300

switch config.model_type
    case 'sparse'
        config.nIteration = 10000;
        config.Gamma = learningRate(layer);
        config.T = 10; % iteration for Langevin
    case 'dense'
        config.nIteration = 150; % dense frame could be 100 to 200, for warm start please use 150, for cold start use 200
        config.Gamma = learningRate(layer);
        config.T = 50; % for warm start please use 100, for cold start use 200
        config.finalize_nIteration = 100;
end

switch lower(config.exp_type)
    case 'texture'
        config.T = 50;
        config.nIteration = 300;
    otherwise
end

config.final_iter = 200;

config.forceLearn = false;
config.BatchSize = 128;

run(fullfile(config.matconvv_path, 'matlab', 'vl_setupnn.m'));

net_cpu = load([config.model_path, config.model_name]);
config.sx = net_cpu.normalization.imageSize(1);
config.sy = net_cpu.normalization.imageSize(2);
net_cpu.layers = net_cpu.layers(1:layer);
net = vl_simplenn_move(net_cpu, 'gpu');


% determine # of filters
img = randn(net.normalization.imageSize, 'single');
res = vl_simplenn(net, gpuArray(img));

config.numFilters = numel(res(end).x);
if strcmp(net_cpu.layers{end}.type, 'conv') == true
    config.numFilterTypes = size(net_cpu.layers{end}.weights{2},2);
else
    for l = layer-1:-1:1
        if strcmp(net_cpu.layers{l}.type, 'conv') == true
            config.numFilterTypes = size(net_cpu.layers{l}.weights{2},2);
            break;
        end
    end
end
config.dydz_sz = size(res(end).x);

net = vl_simplenn_move(net, 'cpu');

