function net = train_model_generative(config, net, syn_mat, imgCell)

learningRate_array = repmat(ones(1,60, 'single')*0.01, floor(config.nIteration / 60),1) * 100; %logspace(-2, -4, 60)
learningRate_array = reshape(learningRate_array, 1, []);

opts.train.weightDecay = 0.0001 ;

opts.train.numEpochs = 1;

opts.train.batchSize = 4;
opts.train.continue = true;
opts.train.gpus = [1] ;

net.filterSelected = 1:prod(config.dydz_sz);
net.selectedLambdas = ones(1, prod(config.dydz_sz), 'single') * 1;

interval = ceil(config.nIteration / 100);

SSD = zeros(config.nIteration, 1);
Energy = zeros(config.nIteration, 1);

mean_img = gather(net.normalization.averageImage);

model_file = [config.working_folder, config.categoryName, '_iter_',...
    num2str(config.nIteration) ,'_model.mat'];
syn_file = [config.working_folder, config.categoryName, '_iter_',...
    num2str(config.nIteration) ,'_syn.mat'];

img_mat = imgcell2mat(imgCell);
imdb = convert2imdb(img_mat);

h = figure;

if ~exist(model_file, 'file') || config.forceLearn == true
    %% learning iteration: Generative learning
    for iter = 1:config.nIteration
        disp( [ 'iteration: ' num2str(iter)]);
        
        %% MLE: Sampling and compute parameters learning
        fprintf('Langevin dynamics sampling...\n');
        syn_mat = langevin_dynamics_fast(config, net, syn_mat);
        fprintf('Langevin finished\n');
        
        
        %% learning
        opts.train.learningRate = learningRate_array(min(iter, numel(learningRate_array))) * config.Gamma;
        net = cnn_generative_train(net, imdb, syn_mat, @getBatch, config, opts.train);
        
        rHat = filter_response(net, img_mat);
        rModel = filter_response(net, syn_mat);
        
%         thisLR = learningRate_array(min(iter, numel(learningRate_array))) * config.Gamma / 10^1;
%         net.selectedLambdas = net.selectedLambdas + ...
%             thisLR * (rHat - rModel);
        
%         fprintf('max gradient is %f, learning rate is %f\n', max(rHat - rModel),thisLR);
        
        Energy(iter) = sum(rModel .* net.selectedLambdas);
        % second part
        Energy(iter) = Energy(iter) ...
            + sum(gather(reshape(syn_mat.^2, [], 1))) / 2 / config.refsig /config.refsig;
        
        SSD(iter) = mean(abs(rHat - rModel));
        
        if mod(iter - 1, interval) == 0 || iter == config.nIteration
            % save samples
            draw_figures(config, syn_mat, iter, mean_img, SSD, Energy);
        end
        
    end
end
end


function imdb = convert2imdb(img_mat)
% --------------------------------------------------------------------
numImages = size(img_mat, 4);
imdb.images.data = img_mat ;
imdb.images.set = ones(1, numImages);
imdb.meta.sets = {'train', 'val', 'test'} ;
end

function im = getBatch(imdb, batch)
% --------------------------------------------------------------------
im = imdb.images.data(:,:,:,batch) ;
end

function [] = draw_figures(config, syn_mat, iter, mean_img, SSD, Energy)
I_syn = convert_syns_mat(config, mean_img, syn_mat);
im = im2uint8(I_syn);
[imind,cm] = rgb2ind(im,256);
% save samples
if iter == 1
    imwrite(imind, cm, [config.Synfolder, 'animation', '.gif'], 'DelayTime', 0.10, 'Loopcount', inf);
else
    imwrite(imind, cm, [config.Synfolder, 'animation', '.gif'], 'WriteMode', 'append', 'DelayTime', 0.10);
end
imwrite(I_syn,[config.Synfolder, num2str(iter, 'dense_original_%04d'), '.png']);
subplot(2, 1, 1);
plot(1:iter, SSD(1:iter), 'r', 'LineWidth', 3);
axis([min(iter, 5), iter+1, 0,  max(SSD(min(iter, 5):end)) * 1.2]);
title('Matched statistics')
drawnow;

subplot(2, 1, 2);
plot(1:iter, Energy(1:iter), 'r', 'LineWidth', 3);
min_ax = min(Energy(min(iter, 5):end)) - abs(min(Energy(min(iter, 5):end))) * 0.2;
max_ax = max(Energy(min(iter, 5):end)) + abs(max(Energy(min(iter, 5):end))) * 0.2;
axis([min(iter, 5), iter+1, min_ax,  max_ax]);
title('Energy')
drawnow;

disp(['SSD: ', num2str(SSD(iter)), ', Energy: ', num2str(Energy(iter))]);
end
